import { Event } from "../../../base/common/event.js";
export interface IUpdate {
    version: string;
    productVersion?: string;
    timestamp?: number;
    url?: string;
    sha256hash?: string;
}
export declare enum StateType {
    Uninitialized = "uninitialized",
    Idle = "idle",
    Disabled = "disabled",
    CheckingForUpdates = "checking for updates",
    AvailableForDownload = "available for download",
    Downloading = "downloading",
    Downloaded = "downloaded",
    Updating = "updating",
    Ready = "ready"
}
export declare enum UpdateType {
    Setup = 0,
    Archive = 1,
    Snap = 2
}
export declare enum DisablementReason {
    NotBuilt = 0,
    DisabledByEnvironment = 1,
    ManuallyDisabled = 2,
    MissingConfiguration = 3,
    InvalidConfiguration = 4,
    RunningAsAdmin = 5
}
export type Uninitialized = {
    type: StateType.Uninitialized;
};
export type Disabled = {
    type: StateType.Disabled;
    reason: DisablementReason;
};
export type Idle = {
    type: StateType.Idle;
    updateType: UpdateType;
    error?: string;
};
export type CheckingForUpdates = {
    type: StateType.CheckingForUpdates;
    explicit: boolean;
};
export type AvailableForDownload = {
    type: StateType.AvailableForDownload;
    update: IUpdate;
};
export type Downloading = {
    type: StateType.Downloading;
};
export type Downloaded = {
    type: StateType.Downloaded;
    update: IUpdate;
};
export type Updating = {
    type: StateType.Updating;
    update: IUpdate;
};
export type Ready = {
    type: StateType.Ready;
    update: IUpdate;
};
export type State = Uninitialized | Disabled | Idle | CheckingForUpdates | AvailableForDownload | Downloading | Downloaded | Updating | Ready;
export declare const State: {
    Uninitialized: Uninitialized;
    Disabled: (reason: DisablementReason) => Disabled;
    Idle: (updateType: UpdateType, error?: string) => Idle;
    CheckingForUpdates: (explicit: boolean) => CheckingForUpdates;
    AvailableForDownload: (update: IUpdate) => AvailableForDownload;
    Downloading: Downloading;
    Downloaded: (update: IUpdate) => Downloaded;
    Updating: (update: IUpdate) => Updating;
    Ready: (update: IUpdate) => Ready;
};
export interface IAutoUpdater extends Event.NodeEventEmitter {
    setFeedURL(url: string): void;
    checkForUpdates(): void;
    applyUpdate?(): Promise<void>;
    quitAndInstall(): void;
}
